@@ -1,7 +1,7 @@
 '''
 
-Defines classes for dealing with fixations, most notably the
-`FixationSequence` object.
+Defines the `Fixation` and `FixationSequence` objects, which are used to
+represent fixation data.
 
 '''
 
@@ -77,7 +77,9 @@ class Fixation:
 	@property
 	def tuple(self):
 		'''Tuple representation of the fixation.'''
-		return (self._x, self._y, self._duration, self._discarded)
+		if self.discarded:
+			return (self._x, self._y, self._duration, True)
+		return (self._x, self._y, self._duration)
 
 
 class FixationSequence:
@@ -103,7 +105,13 @@ class FixationSequence:
 			self.append(fixation)
 
 	def __repr__(self):
-		return 'FixationSequence[%s, ..., %s]' % (str(self._sequence[0]), str(self._sequence[-1]))
+		if len(self) > 2:
+			return f'FixationSequence[{str(self._sequence[0])}, ..., {str(self._sequence[-1])}]'
+		elif len(self) == 2:
+			return f'FixationSequence[{str(self._sequence[0])}, {str(self._sequence[1])}]'
+		elif len(self) == 1:
+			return f'FixationSequence[{str(self._sequence[0])}]'
+		return 'FixationSequence[]'
 
 	def __len__(self):
 		return len(self._sequence)
