@@ -1,62 +1,14 @@
 '''
 
-Defines classes for dealing with texts, most notably the
-`TextBlock` and `InterestArea` objects.
+Defines the `TextBlock` and `InterestArea` objects for handling texts.
 
 '''
 
 
 import re as _re
 import numpy as _np
-from PIL import ImageFont as _ImageFont
-
-
-ALPHABETS = {
-	'Dutch': 'A-Za-z',
-	'English': 'A-Za-z',
-	'French': 'A-ZÇÉÀÈÙÂÊÎÔÛËÏÜŸŒa-zçéàèùâêîôûëïüÿœ',
-	'German': 'A-ZÄÖÜa-zäöüß',
-	'Greek': 'ΑΒΓΔΕΖΗΘΙΚΛΜΝΞΟΠΡΣΣΤΥΦΧΨΩΆΈΉΊΌΎΏΪΫΪ́Ϋ́αβγδεζηθικλμνξοπρσςτυφχψωάέήίόύώϊϋΐΰ',
-	'Italian': 'A-ZÀÉÈÍÌÓÒÚÙa-zàéèíìóòúù',
-	'Polish': 'A-ZĄĆĘŁŃÓŚŹŻa-ząćęłńóśźż',
-	'Portuguese': 'A-ZÁÂÃÀÇÉÊÍÓÔÕÚa-záâãàçéêíóôõú',
-	'Russian': 'АБВГДЕЁЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЬЭЮЯабвгдеёжзийклмнопрстуфхцчшщъыьэюя',
-	'Spanish': 'A-ZÑÁÉÍÓÚÜa-zñáéíóúü'
-}
-_ZONE_REGEX = _re.compile(r'(\[(.+?)\]\{(.+?)\})')
-_ALPHABET = 'A-Za-z'
-_ALPHA = _re.compile(f'[{_ALPHABET}]')
-_ALPHA_PLUS = _re.compile(f'[{_ALPHABET}]+')
-
-
-def set_default_alphabet(alphabet):
-	'''
-
-	By default, Eyekit considers the alphabet to be the standard 26 Latin
-	characters in upper and lower case. If you are working with another language
-	it may be useful to change this default, so that all new `TextBlock` objects
-	are automatically initialized with a particular alphabet. To do this, call
-	`set_default_alphabet()` at the top of your script. For German, for example,
-	you might do this:
-
-	```
-	eyekit.set_default_alphabet('A-Za-zßÄÖÜäöü')
-	```
-
-	Eyekit also provides built-in alphabets for several European languages, so
-	it's also possible to do this:
-
-	```
-	eyekit.set_default_alphabet(eyekit.ALPHABETS['Polish'])
-	```
-
-	'''
-	global _ALPHABET, _ALPHA, _ALPHA_PLUS
-	if not isinstance(alphabet, str):
-		raise ValueError('Invalid alphabet. Should be a string of acceptable characters, e.g."A-Za-zßÄÖÜäöü".')
-	_ALPHABET = alphabet
-	_ALPHA = _re.compile(f'[{_ALPHABET}]')
-	_ALPHA_PLUS = _re.compile(f'[{_ALPHABET}]+')
+import cairocffi as _cairo
+from . import _alpha
 
 
 class Box(object):
@@ -150,6 +102,7 @@ class Character(Box):
 
 	@property
 	def baseline(self):
+		'''*float* The y position of the character baseline'''
 		return self._baseline
 
 
@@ -158,14 +111,14 @@ class InterestArea(Box):
 	'''
 
 	Representation of an interest area – a portion of a `TextBlock` object that
-	is of potenital interest. It is not usually necessary to create
+	is of potential interest. It is not usually necessary to create
 	`InterestArea` objects manually; they are created automatically when
 	you slice a `TextBlock` object or when you iterate over lines, words,
 	characters, ngrams, or zones parsed from the raw text.
 
 	'''
 
-	def __init__(self, chars, label=None, padding=0):
+	def __init__(self, chars, id, padding=0):
 		for char in chars:
 			if not isinstance(char, Character):
 				raise ValueError('chars must only contain Character objects')
@@ -174,10 +127,10 @@ class InterestArea(Box):
 		self._x_br = chars[-1].x_br + padding
 		self._y_br = chars[-1].y_br
 		self._chars = chars
-		self.label = label
+		self._id = str(id)
 
 	def __repr__(self):
-		return 'InterestArea[%s]' % self.label
+		return f'InterestArea[{self.id}, {self.text}]'
 
 	def __getitem__(self, key):
 		return self._chars[key]
@@ -189,27 +142,25 @@ class InterestArea(Box):
 		for char in self._chars:
 			yield char	
 
+	@property
+	def id(self):
+		'''*str* Interest area ID. By default, these ID's have the form 1:5:10, which represents the line number and column indices of the `InterestArea` in its parent `TextBlock`. However, IDs can also be changed to any arbitrary string.'''
+		return self._id
+
+	@id.setter
+	def id(self, id):
+		self._id = str(id)
+
 	@property
 	def baseline(self):
+		'''*float* The y position of the text baseline'''
 		return self._chars[0].baseline
 
 	@property
 	def text(self):
-		'''*str* String represention of the interest area'''
+		'''*str* String representation of the interest area'''
 		return ''.join(map(str, self._chars))
 
-	@property
-	def label(self):
-		'''*str* Arbitrary label for the interest area'''
-		return self._label
-
-	@label.setter
-	def label(self, label):
-		if label is None:
-			self._label = 'slice'
-		else:
-			self._label = str(label)
-
 
 class TextBlock(Box):
 
@@ -220,16 +171,54 @@ class TextBlock(Box):
 
 	'''
 
-	def __init__(self, text, position, font_name, font_size, line_height=None, adjust_bbox=None, alphabet=None):
+	_default_position = (0, 0)
+	_default_font_face = 'Courier New'
+	_default_font_size = 20.0
+	_default_line_height = None
+	_default_alphabet = None
+	
+	_alpha_solo = _re.compile(f'[{_alpha.characters}]')
+	_alpha_plus = _re.compile(f'[{_alpha.characters}]+')
+	_zone_markup = _re.compile(r'(\[(.+?)\]\{(.+?)\})')
+
+	@classmethod
+	def defaults(cls, position=None, font_face=None, font_size=None, line_height=None, alphabet=None):
+		'''
+
+		Set default `TextBlock` parameters. If you plan to create several
+		`TextBlock`s with the same parameters, it may be useful to set the default
+		parameters at the top of your script or at the start of your session:
+
+		```python
+		import eyekit
+		eyekit.TextBlock.defaults(font_face='Helvetica')
+		txt = eyekit.TextBlock('The quick brown fox')
+		print(txt.font_face) # 'Helvetica'
+		```
+		
+		'''
+		if position is not None:
+			cls._default_position = (float(position[0]), float(position[1]))
+		if font_face is not None:
+			cls._default_font_face = str(font_face)
+		if font_size is not None:
+			cls._default_font_size = float(font_size)
+		if line_height is not None:
+			cls._default_line_height = float(line_height)
+		if alphabet is not None:
+			cls._default_alphabet = str(alphabet)
+			cls._alpha_solo = _re.compile(f'[{cls._default_alphabet}]')
+			cls._alpha_plus = _re.compile(f'[{cls._default_alphabet}]+')
+
+	def __init__(self, text, position=None, font_face=None, font_size=None, line_height=None, alphabet=None):
 		'''Initialized with:
 
-		- ```text``` *str* (single line) | *list* of *str* (multiline) : The line or lines of text. Optionally, these can be marked up with arbitrary interest areas (or zones); for example, ```The quick brown fox jump[ed]{past-suffix} over the lazy dog```.
+		- ```text``` *str* (single line) | *list* of *str* (multiline) : The line or lines of text. Optionally, these can be marked up with arbitrary interest areas (zones); for example, ```The quick brown fox jump[ed]{past-suffix} over the lazy dog```.
 		- `position` *tuple*[*float*, *float*] : XY-coordinates of the top left corner of the `TextBlock`'s bounding box.
-		- `font_name` *str* : Name of a font available on your system. Eyekit can access TrueType fonts that are discoverable by Pillow.
+		- `font_face` *str* : Name of a font available on your system.
 		- `font_size` *float* : Font size in points (at 72dpi). To convert a font size from some other dpi, use `eyekit.tools.font_size_at_72dpi()`.
 		- `line_height` *float* : Height of a line of text in points. Generally speaking, for single line spacing, the line height is equal to the font size, for double line spacing, the light height is equal to 2 × the font size, etc. By default, the line height is assumed to be the same as the font size (single line spacing). This parameter also effectively determines the height of the bounding boxes around interest areas.
-		- `adjust_bbox` *float* : Pixel adjustment to the y-position of bounding boxes relative to the font baseline. Some fonts, such as Courier and Helvetica have quite high baselines making the bounding boxes a little low relative to the text. This parameter can be used to adjust this.
-		- `alphabet` *str* : A string of characters that are considered alphabetical. This is case sensitive, so you must supply upper- and lower-case variants. By default, the alphabet is set to `A-Za-z`, but for German, for example, you might use this: `A-Za-zßÄÖÜäöü`. Eyekit also provides built-in alphabets for several European languages, for example, `eyekit.ALPHABETS['French']`. If you are creating many `TextBlock` objects with the same alphabet, it may be preferable to use `set_default_alphabet()`.
+		- `alphabet` *str* : A string of characters that are to be considered alphabetical, which determines, for example, what is considered a word. By default, Eyekit considers the standard Latin, Greek, and Cyrillic alphabets to be alphabetical, plus the special and accented characters from most European languages. However, if you need support for some other alphabet, or if you want to modify Eyekit's default behavior, you can set an alternative alphabet here. This parameter is case sensitive, so you must supply upper- and lower-case variants. For example, if you wanted to treat apostrophes and hyphens as alphabetical, you might use `alphabet="A-Za-z'-"`. This would allow, for example, "Where's the orang-utan?" to be treated as three words rather than five.
 		'''
 
 		# TEXT
@@ -241,72 +230,77 @@ class TextBlock(Box):
 			raise ValueError('text should be a string or a list of strings')
 
 		# POSITION
-		try:
+		if position is None:
+			self._x_tl = self._default_position[0]
+			self._first_baseline = self._default_position[1]
+		else:
 			self._x_tl = float(position[0])
-			self._y_tl = float(position[1])
-		except:
-			raise ValueError('position should be tuple representing the XY coordinates of the top left corner of the TextBlock')
+			self._first_baseline = float(position[1])
 
-		# FONT NAME AND SIZE
-		self._font_name = str(font_name)
-		self._font_size = float(font_size)
+		# FONT FACE
+		if font_face is None:
+			self._font_face = self._default_font_face
+		else:
+			self._font_face = str(font_face)
+
+		# FONT SIZE
+		if font_size is None:
+			self._font_size = self._default_font_size
+		else:
+			self._font_size = float(font_size)
 
 		# LINE HEIGHT
 		if line_height is None:
-			self._line_height = self._font_size
+			if self._default_line_height is None:
+				self._line_height = self._font_size
+			else:
+				self._line_height = self._default_line_height
 		else:
 			self._line_height = float(line_height)
 
-		# BOUNDING BOX ADJUSTMENT
-		if adjust_bbox is None:
-			self._adjust_bbox = 0
-		else:
-			self._adjust_bbox = float(adjust_bbox)
-
 		# ALPHABET
-		self.alphabet = alphabet
+		if alphabet is None:
+			if self._default_alphabet is None:
+				self._alphabet = None
+			else:
+				self._alphabet = self._default_alphabet
+		else:
+			self._alphabet = str(alphabet)
+			self._alpha_solo = _re.compile(f'[{self._alphabet}]')
+			self._alpha_plus = _re.compile(f'[{self._alphabet}]+')
 
 		# LOAD FONT
-		try:
-			self._font = _ImageFont.truetype(self._font_name, int(self._font_size * 64)).font # font_size must be int, so we'll work with a much larger font size and scale the widths back down later for greater precision
-		except:
-			raise ValueError(f'Unable to load font "{self._font_name}" in size {self._font_size}.')
-		self._first_baseline = self._y_tl + (self._font.ascent - self._font.descent) / 64
-		self._half_space_width = self._font.getsize(' ')[0][0] / 128
+		self._font = _load_font(self._font_face, self._font_size)
+		self._x_height = self._font.text_extents('x')[3]
+		self._half_space = self._font.text_extents(' ')[4] / 2
 
 		# INITIALIZE CHARACTERS AND ZONES
 		self._characters, self._zones = self._initialize_text_block()
 
-		# SET BOTTOM-RIGHT CORNER OF BOUNDING BOX
+		# SET REMAINING TEXTBLOCK COORDINATES
 		self._x_br = max([line[-1].x_br for line in self._characters])
+		self._y_tl = self._characters[0][0].y_tl
 		self._y_br = self._y_tl + self.n_rows * self._line_height
 
 	def __repr__(self):
 		return 'TextBlock[%s...]' % ''.join(map(str, self._characters[0][:16]))
 
-	def __getitem__(self, key):
+	def __getitem__(self, id):
 		'''
-		Subsetting a TextBlock object with a row,column index returns
-		the indexed characters as an InterestArea.
+		Subsetting a TextBlock object with a key of the form x:y:z returns
+		characters y to z on row x as an InterestArea.
 		'''
-		if self.n_rows == 1 and (isinstance(key, int) or isinstance(key, slice)):
-			key = 0, key
-		if not isinstance(key, tuple) or not len(key) == 2:
-			raise IndexError('Index to multiline text should specify both the row and column')
-		r, c = key
-		if not isinstance(r, int) or r >= self.n_rows or r < 0:
-			raise IndexError('Invalid row index')
-		if isinstance(c, int):
-			if c < 0 or c >= self.n_cols:
-				raise IndexError('Invalid column index')
-			return InterestArea([self._characters[r][c]])
-		if isinstance(c, slice):
+		if isinstance(id, str):
+			if id in self._zones:
+				return self._zones[id]
+			rse = id.split(':')
 			try:
-				return InterestArea(self._characters[r][c])
-			except IndexError as exception:
-				exception.args = ('Invalid column slice',)
-				raise
-		raise IndexError('Invalid index to TextBlock object')
+				r, s, e = int(rse[0]), int(rse[1]), int(rse[2])
+			except:
+				raise KeyError('Invalid InterestArea ID')
+		elif isinstance(id, slice):
+			r, s, e = id.start, id.stop, id.step
+		return InterestArea(self._characters[r][s:e], f'{r}:{s}:{e}')
 
 	def __len__(self):
 		return sum([len(line) for line in self._characters])
@@ -322,13 +316,18 @@ class TextBlock(Box):
 
 	@property
 	def text(self):
-		'''*str* String represention of the text`'''
+		'''*str* String representation of the text`'''
 		return ' '.join([''.join(map(str, line)) for line in self._characters])
 
 	@property
-	def font_name(self):
+	def position(self):
+		'''*tuple* Position of the `TextBlock`'''
+		return self._x_tl, self._y_tl
+
+	@property
+	def font_face(self):
 		'''*str* Name of the font'''
-		return self._font_name
+		return self._font_face
 
 	@property
 	def font_size(self):
@@ -340,26 +339,10 @@ class TextBlock(Box):
 		'''*float* Line height in points'''
 		return self._line_height
 
-	@property
-	def adjust_bbox(self):
-		'''*float* Line height in points'''
-		return self._adjust_bbox
-
 	@property
 	def alphabet(self):
 		'''*str* Characters that are considered alphabetical'''
 		return self._alphabet
-	
-	@alphabet.setter
-	def alphabet(self, alphabet):
-		if alphabet:
-			self._alphabet = alphabet
-			self._alpha = _re.compile(f'[{alphabet}]')
-			self._alpha_plus = _re.compile(f'[{alphabet}]+')
-		else:
-			self._alphabet = _ALPHABET
-			self._alpha = _ALPHA
-			self._alpha_plus = _ALPHA_PLUS
 
 	@property
 	def n_rows(self):
@@ -406,16 +389,6 @@ class TextBlock(Box):
 				return zone
 		return None
 
-	def get_zone(self, label):
-		'''
-
-		Retrieve a marked-up zone by its label.
-		
-		'''
-		if label not in self._zones:
-			raise KeyError('There is no zone with the label %s' % label)
-		return self._zones[label]
-
 	def lines(self):
 		'''
 
@@ -423,7 +396,7 @@ class TextBlock(Box):
 
 		'''
 		for r, line in enumerate(self._characters):
-			yield InterestArea(line, 'line_%i'%r)
+			yield InterestArea(line, f'{r}:{0}:{len(line)}')
 
 	def which_line(self, fixation):
 		'''
@@ -442,7 +415,7 @@ class TextBlock(Box):
 		Iterate over each word as an `InterestArea`. Optionally, you can
 		supply a regex pattern to define what constitutes a word or to pick
 		out specific words. For example, `r'\\b[Tt]he\\b'` gives you all
-		occurances of the word *the* or `'[a-z]+ing'` gives you all words
+		occurrences of the word *the* or `'[a-z]+ing'` gives you all words
 		ending with *-ing*. `add_padding` adds half of the width of a space
 		character to the left and right edges of the word's bounding box, so
 		that fixations that fall on a space between two words will at least
@@ -454,7 +427,7 @@ class TextBlock(Box):
 		else:
 			pattern = _re.compile(pattern)
 		if add_padding:
-			padding = self._half_space_width
+			padding = self._half_space
 		else:
 			padding = 0
 		word_i = 0
@@ -463,9 +436,10 @@ class TextBlock(Box):
 				continue
 			line_str = ''.join(map(str, line))
 			for word in pattern.findall(line_str):
-				c = line_str.find(word)
+				s = line_str.find(word)
+				e = s + len(word)
 				line_str = line_str.replace(word, '#'*len(word), 1)
-				yield InterestArea(self._characters[r][c:c+len(word)], 'word_%i'%word_i, padding=padding)
+				yield InterestArea(self._characters[r][s:e], f'{r}:{s}:{e}', padding=padding)
 				word_i += 1
 
 	def which_word(self, fixation, pattern=None, line_n=None, add_padding=True):
@@ -491,10 +465,10 @@ class TextBlock(Box):
 		for r, line in enumerate(self._characters):
 			if line_n is not None and r != line_n:
 				continue
-			for char in line:
-				if alphabetical_only and not self._alpha.match(str(char)):
+			for s, char in enumerate(line):
+				if alphabetical_only and not self._alpha_solo.match(str(char)):
 					continue
-				yield InterestArea([char], 'character_%i'%char_i)
+				yield InterestArea([char], f'{r}:{s}:{s+1}')
 				char_i += 1
 
 	def which_character(self, fixation, line_n=None, alphabetical_only=True):
@@ -509,7 +483,7 @@ class TextBlock(Box):
 				return character
 		return None
 
-	def ngrams(self, n, line_n=None, alphabetical_only=True, yield_rc=False):
+	def ngrams(self, n, line_n=None, alphabetical_only=True):
 		'''
 
 		Iterate over each ngram, for given n, as an `InterestArea`.
@@ -519,14 +493,12 @@ class TextBlock(Box):
 		for r, line in enumerate(self._characters):
 			if line_n is not None and r != line_n:
 				continue
-			for c in range(len(line)-(n-1)):
-				if alphabetical_only and not self._alpha_plus.fullmatch(''.join(map(str, line[c:c+n]))):
+			for s in range(len(line)-(n-1)):
+				e = s + n
+				if alphabetical_only and not self._alpha_plus.fullmatch(''.join(map(str, line[s:e]))):
 					continue
-				ngram = InterestArea(line[c:c+n], '%igram_%i'%(n, ngram_i))
-				if yield_rc:
-					yield ngram, (r, c)
-				else:
-					yield ngram
+				ngram = InterestArea(line[s:e], f'{r}:{s}:{e}')
+				yield ngram
 				ngram_i += 1
 
 	# No which_ngram() method because, by definition, a fixation is
@@ -539,11 +511,20 @@ class TextBlock(Box):
 	def _serialize(self):
 		'''
 		
-		Returns the `TextBlock`'s ininialization arguments as a dictionary
+		Returns the `TextBlock`'s initialization arguments as a dictionary
 		for serialization.
 		
 		'''
-		return {'position': (self.x_tl, self.y_tl), 'font_name': self.font_name, 'font_size': self.font_size, 'line_height': self.line_height, 'adjust_bbox':self.adjust_bbox, 'alphabet': self.alphabet, 'text': self._text}
+		data = {
+			'position': (self.x_tl, self._first_baseline),
+			'font_face': self.font_face,
+			'font_size': self.font_size,
+			'line_height': self.line_height
+		}
+		if self.alphabet is not None:
+			data['alphabet'] = self.alphabet
+		data['text'] = self._text
+		return data
 
 	def _initialize_text_block(self):
 		'''
@@ -555,38 +536,49 @@ class TextBlock(Box):
 		'''
 		characters = []
 		zones = {}
-		y_tl = self.y_tl - (self._line_height - self._font_size) / 2 + self._adjust_bbox # y_tl of character bounding boxes on first lines
 		baseline = self._first_baseline
+		midline = baseline - self._x_height / 2 # midline is half an x-height higher than the baseline
+		y_tl = midline - self._line_height / 2 # top of bounding box is half a line height above the midline
 		for r, line in enumerate(self._text):
 			# Parse and strip out interest area zones from this line
-			for zone_markup, zone_text, zone_label in _ZONE_REGEX.findall(line):
-				if zone_label in zones:
-					raise ValueError('The zone label %s has been used more than once.' % zone_label)
+			for zone_markup, zone_text, zone_id in self._zone_markup.findall(line):
+				if zone_id in zones:
+					raise ValueError('The zone ID "%s" has been used more than once.' % zone_id)
 				c = line.find(zone_markup)
-				zones[zone_label] = (r, c, len(zone_text)) # record row index, column index, and length of zone
+				zones[zone_id] = (r, c, len(zone_text)) # record row index, column index, and length of zone
 				line = line.replace(zone_markup, zone_text) # replace the marked up zone with the unmarked up text
 			# Create the set of Character objects for this line
 			characters_line = []
 			x_tl = self.x_tl # x_tl of first character bounding box on this line
-			y_br = y_tl + self._line_height # y_br of all character bounding boes on this line
-			line_width = self._font.getsize(line)[0][0] # anticipate the length of the line
-			words = line.split(' ')
-			word_widths = [(word + ' ', self._font.getsize(word + ' ')[0][0]) for word in words[:-1]]
-			word_widths.append((words[-1], self._font.getsize(words[-1])[0][0])) # calculate word widths - all words include a trailing space except the last word in the line
-			line_scaling_factor = line_width / sum([width for _, width in word_widths]) # calculate scaling factor needed to scale down the word widths so that their sum width matches the anticipated line width
-			for word, word_width in word_widths:
-				word_width *= line_scaling_factor # scale down the word width
-				char_widths = [self._font.getsize(char)[0][0] for char in word]
-				word_scaling_factor = word_width / sum(char_widths) # calculate a scaling factor for scaling down character widths so that their sum width matches the target word length
-				for char, char_width in zip(word, char_widths):
-					char_width *= word_scaling_factor
-					char_width /= 64
-					characters_line.append(Character(char, x_tl, y_tl, x_tl+char_width, y_br, baseline))
-					x_tl += char_width
+			y_br = y_tl + self._line_height # y_br of all character bounding boxes on this line
+			for char in line:
+				char_width = self._font.text_extents(char)[4]
+				characters_line.append(Character(char, x_tl, y_tl, x_tl+char_width, y_br, baseline))
+				x_tl += char_width
 			characters.append(characters_line)
 			y_tl += self._line_height
 			baseline += self._line_height
 		# Set up and store the zoned interest areas based on the indices stored earlier.
-		for zone_label, (r, c, length) in zones.items(): # Needs to be done in two steps because IAs can't be computed until character positions are known
-			zones[zone_label] = InterestArea(characters[r][c:c+length], zone_label)
+		for zone_id, (r, c, length) in zones.items(): # Needs to be done in two steps because IAs can't be created until character positions are known
+			zones[zone_id] = InterestArea(characters[r][c:c+length], zone_id)
 		return characters, zones
+
+def _memoize(f):
+	memo = {}
+	def helper(x, y):
+		if (x, y) not in memo:
+			memo[(x, y)] = f(x, y)
+		return memo[(x, y)]
+	return helper
+
+@_memoize
+def _load_font(font_face, font_size):
+	'''
+
+	Load a ScaledFont object from Cairo for use in calculating text extents.
+
+	'''
+	context = _cairo.Context(_cairo.ImageSurface(_cairo.FORMAT_ARGB32, 1, 1))
+	context.select_font_face(font_face)
+	context.set_font_size(font_size)
+	return _cairo.ScaledFont(context.get_font_face(), context.get_font_matrix())
