@@ -6,6 +6,8 @@ bounds fixations and snapping fixations to the lines of text.
 '''
 
 
+from os.path import splitext as _splittext
+import cairocffi as _cairo
 from ._core import distance as _distance
 from .fixation import FixationSequence as _FixationSequence
 from .text import TextBlock as _TextBlock
@@ -74,38 +76,48 @@ def fixation_sequence_distance(fixation_sequence1, fixation_sequence2):
 	cost, _ = _drift._dynamic_time_warping(fixation_sequence1.XYarray(), fixation_sequence2.XYarray())
 	return cost
 
-def align_to_screenshot(text_block, screenshot_path, output_path=None, show_bounding_boxes=False):
+def align_to_screenshot(text_block, screenshot_path, output_path=None, show_text=True, show_bounding_boxes=False):
 	'''
 
-	Create an image dipicting a screenshot overlaid with a
-	`eyekit.text.TextBlock` in green. The output is saved to the same directory
-	as the screenshot file. This is useful for establishing the correct
-	`eyekit.text.TextBlock` parameters to match what participants are actually
-	seeing.
+	Create an image depicting a PNG screenshot overlaid with a
+	`eyekit.text.TextBlock` in green. If no output path is provided, the output
+	is written to the same directory as the screenshot file. This is useful for
+	establishing the correct `eyekit.text.TextBlock` parameters to match what
+	participants are actually seeing.
 
 	'''
-	from os.path import abspath as _abspath, splitext as _splittext
-	from PIL import Image as _PILImage
-	from .image import Image as _Image
-	screenshot_path = _abspath(screenshot_path)
-	screenshot = _PILImage.open(screenshot_path)
-	screen_width, screen_height = screenshot.size
-	img = _Image(screen_width, screen_height)
-	img.reference_raster_image(screenshot_path, 0, 0, screen_width, screen_height)
+	surface = _cairo.ImageSurface(_cairo.FORMAT_ARGB32, 1, 1).create_from_png(screenshot_path)
+	context = _cairo.Context(surface)
+	screen_width = surface.get_width()
+	screen_height = surface.get_height()
+	context.set_source_rgb(0.60392, 0.80392, 0.19607)
+	context.select_font_face(text_block.font_face)
+	context.set_font_size(text_block.font_size)
+	context.set_line_width(2)
+	context.move_to(text_block.x_tl, 0)
+	context.line_to(text_block.x_tl, screen_height)
+	context.stroke()
+	for i, line in enumerate(text_block.lines()):
+		if i == 0:
+			context.move_to(0, text_block._first_baseline)
+			context.line_to(screen_width, text_block._first_baseline)
+			context.stroke()
+			context.set_dash([8, 4])
+		else:
+			context.move_to(text_block.x_tl, line.baseline)
+			context.line_to(screen_width, line.baseline)
+			context.stroke()
+		if show_text:
+			context.move_to(line.x_tl, line.baseline)
+			context.show_text(line.text)
 	if show_bounding_boxes:
+		context.set_dash([])
 		for word in text_block.words(add_padding=False):
-			img.draw_rectangle(word.box, color='#85C01E')
-	else:
-		img.render_text(text_block, color='#85C01E')
-	img.draw_rectangle(text_block.box, color='#85C01E', dashed=True)
-	img.draw_line((text_block.x_tl, text_block.y_tl), (text_block.x_tl, 0), color='#85C01E', dashed=True)
-	img.draw_line((text_block.x_tl, text_block.y_tl), (0, text_block.y_tl), color='#85C01E', dashed=True)
-	img.draw_circle(text_block.x_tl, text_block.y_tl, 8, color='#85C01E')
+			context.rectangle(*word.box)
+			context.stroke()
 	if output_path is None:
-		output_path, _ = _splittext(screenshot_path)
-		img.save(output_path + '_eyekit.png')
-	else:
-		img.save(output_path)
+		output_path = _splittext(screenshot_path)[0] + '_eyekit.png'
+	surface.write_to_png(output_path)
 
 def font_size_at_72dpi(font_size, at_dpi=96):
 	'''
@@ -115,4 +127,4 @@ def font_size_at_72dpi(font_size, at_dpi=96):
 	equivalent size at 72dpi.
 	
 	'''
-	return font_size * at_DPI / 72
+	return font_size * at_dpi / 72
